import sys
import argparse
from app.commands.create import createCommand
from app.commands.watch import WatchCommand
from app.config import Config
import logging

logLevelNameToInt = {
    'info': logging.INFO,
    'debug': logging.DEBUG,
    'error': logging.ERROR,
}

def main():
    parser = argparse.ArgumentParser(
        prog='kubelink',
        description='kubelink description',
        epilog='Text at the bottom of help'
    )
    logLevelArgumentHelp = 'Set log level. Default: info. Available levels: info, debug, error'
    parser.add_argument('--log', help=logLevelArgumentHelp, default='info')

    subparsers = parser.add_subparsers(help='sub-command help')

    createp = subparsers.add_parser('create', help='Create a kubelink preset')
    createp.add_argument('--name', help='Name of kubelink preset', required=True)
    createp.add_argument('-s', '--source', help='Local source directory to sync', required=True)
    createp.add_argument('-d', '--destination', help='Destination directory in kubernetes pod', required=True)
    createp.add_argument('--namespace', help='Kubernetes namespace', default="default")
    createp.add_argument('-l', '--selector', help='Label selector to find pod. For example: -s "app=php"', default="", required=True)
    createp.add_argument('-c', '--container', help='Container name. Required if pod contains more than one container', default="")
    createp.add_argument('--log', help=logLevelArgumentHelp, default='info')

    watchp = subparsers.add_parser('watch', help='Watch source directory to any changes and sync it to kubernetes pod')
    watchp.add_argument('name', help='kubelink preset name. If you didn\'t create it yet, you have to run "kubelink create" command')
    watchp.add_argument('--log', help=logLevelArgumentHelp, default='info')

    args = parser.parse_args()

    logLevel = args.log
    if logLevel not in logLevelNameToInt:
        logLevel = 'info'
    logging.basicConfig(level=logLevelNameToInt[logLevel])

    if len(sys.argv) < 2:
        parser.print_help()
        exit()
    else:
        command = sys.argv[1]
    config = Config()

    if command == 'create':
        createCommand(config, args)
    elif command == 'watch':
        watch = WatchCommand(config, args)
        watch.run()
    else:
        logging.error('Invalid command: ' + command)
        exit(1)

if __name__ == "__main__":
    main()


import os
import yaml
import logging

class Config:
    def __init__(self) -> None:
        self.CONFIG_DIR_PATH = self._getConfigDir()
        self.CONFIG_FILENAME = 'config.yaml'
        self.CONFIG_FILE_PATH = os.path.join(self.CONFIG_DIR_PATH, self.CONFIG_FILENAME)

        self._configFolderCheck()

    def savePreset(self, preset: dict) -> bool:
        filteredPreset = self._filterPresetProps(preset)
        logging.debug('Filtered preset: ' + str(filteredPreset))
        config = self._readConfig()
        logging.debug('Read config: ' + str(config))
        if config == None:
            config = {
                'presets': [filteredPreset]
            }
        else:
            self._mergePresets(config, filteredPreset)
            logging.debug('Merged config: ' + str(config))

        return self._writeConfig(config)

    def presetExists(self, presetName: str) -> bool:        
        preset = self.findPreset(presetName)
        return preset != None
    
    def findPreset(self, name: str):
        config = self._readConfig()
        if config == None:
            return False

        if 'presets' not in config:
            return None

        for v in config['presets']:
            if v['name'] == name:
                return v
        return None

    def _getConfigDir(self):
        home = os.path.expanduser('~')
        return os.path.join(home, '.kubelink')

    def _configFolderCheck(self):
        if not os.path.isdir(self.CONFIG_DIR_PATH):
            os.mkdir(self.CONFIG_DIR_PATH)
            logging.debug('Created config directory ' + self.CONFIG_DIR_PATH)

        if not os.path.exists(self.CONFIG_FILE_PATH):
            f = open(self.CONFIG_FILE_PATH, 'w')
            f.write('')
            f.close()
            logging.debug('Created config file ' + self.CONFIG_FILE_PATH)

    def _readConfig(self):
        if not os.path.exists(self.CONFIG_FILE_PATH):
            logging.debug('_readConfig - File not found: ' + self.CONFIG_FILE_PATH)
            return None

        f = open(self.CONFIG_FILE_PATH, 'r')
        data = yaml.load(f, Loader=yaml.FullLoader)
        f.close()
        return data

    def _writeConfig(self, config: dict) -> bool:
        f = open(self.CONFIG_FILE_PATH, 'w')
        yaml.dump(config, f)
        f.close()
        return True

    def _filterPresetProps(self, preset: dict):
        allowedProps = ['name', 'source', 'destination', 'namespace', 'selector', 'container']
        for k in preset:
            if k not in allowedProps:
                del preset[k]
        return preset

    def _mergePresets(self, config: dict, preset: dict):
        # Merge values with existing config
        for v in config['presets']:
            if v['name'] == preset['name']:
                for prop in preset:
                    v[prop] = preset[prop]
                return config
        config['presets'].append(preset)
        return config


import time
from watchdog.observers import Observer
from app.event_handler import EventHandler
import logging
from app.config import Config
from argparse import Namespace
from app.kubernetes import Kubernetes

class WatchCommand:
    def __init__(self, config: Config, args: Namespace) -> None:
        self.config = config
        self.args = args

    def run(self):
        preset = self.config.findPreset(self.args.name)        
        if preset == None:
            logging.error('Preset not found. First you have to run: "kubelink create" to create a preset')
            exit(1)
        
        logging.debug('Found preset: ' + str(preset))

        kube = Kubernetes()

        pods = kube.findPods(preset['selector'], preset['namespace'], preset['container'])
        logging.info('Found ' + str(len(pods)) + ' pods')

        if len(pods) == 0:
            print('Pods to sync not found!')
            exit()

        event_handler = EventHandler(preset, pods)
        observer = Observer()
        observer.schedule(event_handler, preset['source'], recursive=True)
        observer.start()

        logging.info('Watch on changes')

        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            observer.stop()
        observer.join()



from app.config import Config
from argparse import Namespace
import logging

def createCommand(config: Config, args: Namespace):
    preset = {
        'name': args.name,
        'source': args.source,
        'destination': args.destination,
        'namespace': args.namespace,
        'selector': args.selector,
        'container': args.container
    }

    logging.info('Create preset')
    logging.debug('Create preset: ' + str(preset))

    isNew = True
    if config.presetExists(preset['name']):
        isNew = False

    if not config.savePreset(preset):
        logging.error('Failed to save a new preset')
        exit(1)

    if isNew:
        print('Successfully added preset: ' + args.name)
    else:
        print('Successfully updated preset: ' + args.name)



import yaml
import subprocess
import logging

class Kubernetes:
    def __init__(self) -> None:
        pass

    def findPods(self, selector: str, namespace: str, containerName: str = ''):
        command = ['kubectl', 'get', 'pod', '-n', namespace, '--selector', selector, '-o', 'yaml']
        logging.debug('Find pods kubectl command: ' + ' '.join(command))
        result = subprocess.run(command, check=True, text=True, stdout=subprocess.PIPE)
        logging.debug('kubectl find pods result: ' + result.stdout)
        result = yaml.load(result.stdout, yaml.FullLoader)

        if result == None or 'items' not in result:
            #print('Failed to get pods from kubernetes with command: ', command.join(' '))
            return []
        result = result['items']
        #print('result', result)

        if len(result) == 0:
            return []

        pods = []
        for item in result:
            countContainers = len(item['spec']['containers'])
            if countContainers == 0:
                continue
                
            for container in item['spec']['containers']:
                if containerName == '' or container['name'] == containerName:
                    pods.append({
                        'name': item['metadata']['name'],
                        'container': containerName,
                    })

        logging.debug('kubectl get pods: ' + str(pods))
        return pods



from watchdog.events import FileSystemEventHandler
import os
import subprocess
import logging

class EventHandler(FileSystemEventHandler):
    def __init__(self, preset, pods) -> None:
        super().__init__()
        self.preset = preset
        self.pods = pods

    def on_any_event(self,event):
        #print('Recived event : ' + event.event_type + ' - ' + event.src_path)
        pass

    def on_modified(self, event):
        # Skip uploading directory on modified event
        if os.path.isdir(event.src_path):
            return

        self._uploadFile(event.src_path)
    
    def on_created(self, event):
        self._uploadFile(event.src_path)

    def on_deleted(self, event):
        self._deleteFile(event.src_path)

    def _getRelativePath(self, fullLocalPath):
        source = self.preset['source']
        if fullLocalPath == source:
            return None

        if fullLocalPath.find(source) == 0:
            # ok
            path = fullLocalPath[len(source):]
            if path[0] == '/':
                path = path[1:]
            return path
        else:
            logging.error('_getRelativePath - Path is wrong. fullPath: ' + fullLocalPath)
            return None

    def _getRemotePath(self, relativePath):
        return os.path.join(self.preset['destination'], relativePath)

    def _isNeedSkip(self, filePath) -> bool:
        # Ignore PhpStorm temp files
        return len(filePath) < 1 or filePath[-1:] == '~'

    def _uploadFile(self, filePath):
        if self._isNeedSkip(filePath):
            logging.debug('Ignore file: ' + filePath)
            return

        relPath = self._getRelativePath(filePath)
        if relPath == None:
            return
        logging.debug('Relative local path: ' + relPath)

        remotePath = self._getRemotePath(relPath)

        logging.debug('Remote path: ' + remotePath)

        for pod in self.pods:
            remoteUri = self.preset['namespace'] + '/' + pod['name'] + ':' + remotePath
            logging.debug('Remote URI: ' + remoteUri)
            logging.info('Upload file: ' + relPath)
            self._cpCommand(filePath, remoteUri, pod['container'])

    def _deleteFile(self, localFilePath):
        if self._isNeedSkip(localFilePath):
            logging.debug('Ignore file: ' + localFilePath)
            return

        relPath = self._getRelativePath(localFilePath)
        logging.debug('Delete file relative path: ' + str(relPath))
        if relPath == None:
            logging.error('Delete file relative path is None: ' + localFilePath)
            return

        remotePath = self._getRemotePath(relPath)
        logging.info('Delete file: ' + remotePath)

        for pod in self.pods:
            self._deleteCommand(pod['name'], remotePath, pod['container'], self.preset['namespace'])

    def _deleteCommand(self, podName, filepath, container=None, namespace='default'):
        #k exec provider-m-php-b5579ddbb-5v9hn -n production -- sh -c 'rm -rf /code/myfile.txt'
        command = ['kubectl', 'exec', podName, '-n', namespace]
        if container != None and container != '':
            command.append('-c')
            command.append(container)

        command.append('--')
        command.append('sh')
        command.append('-c')
        command.append("rm -rf " + filepath)
        logging.debug('rm command: ' + ' '.join(command))

        result = subprocess.run(command, check=True, text=True, stdout=subprocess.PIPE)
        logging.debug('rm result: ' + result.stdout)
        return result.stdout

    def _cpCommand(self, localPath, remoteUri, container=None):
        if not os.path.exists(localPath):
            logging.debug('Copy file failed - file does not exists: ' + str(localPath))
            return

        # kubectl cp /tmp/foo <some-namespace>/<some-pod>:/tmp/bar
        command = ['kubectl', 'cp', localPath, remoteUri]
        if container != None and container != '':
            command.append('-c')
            command.append(container)

        logging.debug('cp command: ' + ' '.join(command))
        result = subprocess.run(command, check=True, text=True, stdout=subprocess.PIPE)
        logging.debug('cp result: ' + result.stdout)


from setuptools import setup, find_packages
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()

setup(
    name = 'kubelink',
    version = '0.0.3',
    author = 'Volodymyr Tsumanchuk',
    author_email = 'buorn2011@gmail.com',
    license = 'MIT',
    description = 'kubelink automatically sync files to kubernetes pods',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = 'https://github.com/waverage/kubelink',
    py_modules = ['main', 'app', 'app.commands', 'app.config'],
    packages = find_packages(),
    install_requires = [requirements],
    python_requires='>=3.7',
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
    ],
    entry_points = '''
        [console_scripts]
        kubelink=main:main
    '''
)